@@ -47,10 +47,11 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyChat       EmpathyChat;
 typedef struct _EmpathyChatClass  EmpathyChatClass;
+typedef struct _EmpathyChatPriv   EmpathyChatPriv;
 
 struct _EmpathyChat {
 	GtkBin parent;
-	gpointer priv;
+	EmpathyChatPriv *priv;
 
 	/* Protected */
 	EmpathyChatView *view;
@@ -88,6 +89,8 @@ void               empathy_chat_set_show_contacts    (EmpathyChat *chat,
 guint              empathy_chat_get_nb_unread_messages (EmpathyChat   *chat);
 
 void               empathy_chat_messages_read        (EmpathyChat *self);
+
+gboolean           empathy_chat_is_composing (EmpathyChat *chat);
 G_END_DECLS
 
 #endif /* __EMPATHY_CHAT_H__ */
