@@ -39,6 +39,7 @@
 #endif
 
 #include "empathy-contact.h"
+#include "empathy-individual-manager.h"
 #include "empathy-utils.h"
 #include "empathy-enum-types.h"
 #include "empathy-marshal.h"
@@ -69,6 +70,7 @@ typedef struct {
    */
   GHashTable *location;
   GHashTable *groups;
+  gchar **client_types;
 } EmpathyContactPriv;
 
 static void contact_finalize (GObject *object);
@@ -84,6 +86,9 @@ static void update_geocode (EmpathyContact *contact);
 static void empathy_contact_set_location (EmpathyContact *contact,
     GHashTable *location);
 
+static void contact_set_client_types (EmpathyContact *contact,
+    const gchar * const *types);
+
 static void set_capabilities_from_tp_caps (EmpathyContact *self,
     TpCapabilities *caps);
 
@@ -109,7 +114,8 @@ enum
   PROP_HANDLE,
   PROP_CAPABILITIES,
   PROP_IS_USER,
-  PROP_LOCATION
+  PROP_LOCATION,
+  PROP_CLIENT_TYPES
 };
 
 enum {
@@ -162,6 +168,11 @@ tp_contact_notify_cb (TpContact *tp_contact,
     {
       contact_set_avatar_from_tp_contact (EMPATHY_CONTACT (contact));
     }
+  else if (!tp_strdiff (param->name, "client-types"))
+    {
+      contact_set_client_types (EMPATHY_CONTACT (contact),
+          tp_contact_get_client_types (tp_contact));
+    }
 }
 
 static void
@@ -222,6 +233,7 @@ contact_constructed (GObject *object)
   GHashTable *location;
   TpHandle self_handle;
   TpHandle handle;
+  const gchar * const *client_types;
 
   if (priv->tp_contact == NULL)
     return;
@@ -232,6 +244,10 @@ contact_constructed (GObject *object)
   if (location != NULL)
     empathy_contact_set_location (contact, location);
 
+  client_types = tp_contact_get_client_types (priv->tp_contact);
+  if (client_types != NULL)
+    contact_set_client_types (contact, client_types);
+
   set_capabilities_from_tp_caps (contact,
       tp_contact_get_capabilities (priv->tp_contact));
 
@@ -364,6 +380,14 @@ empathy_contact_class_init (EmpathyContactClass *class)
         G_TYPE_HASH_TABLE,
         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (object_class,
+      PROP_CLIENT_TYPES,
+      g_param_spec_boxed ("client-types",
+        "Contact client types",
+        "Client types of the contact",
+        G_TYPE_STRV,
+        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   signals[PRESENCE_CHANGED] =
     g_signal_new ("presence-changed",
                   G_TYPE_FROM_CLASS (class),
@@ -387,6 +411,7 @@ empathy_contact_init (EmpathyContact *contact)
   contact->priv = priv;
 
   priv->location = NULL;
+  priv->client_types = NULL;
   priv->groups = NULL;
 }
 
@@ -403,6 +428,7 @@ contact_finalize (GObject *object)
     g_hash_table_destroy (priv->groups);
   g_free (priv->alias);
   g_free (priv->id);
+  g_strfreev (priv->client_types);
 
   G_OBJECT_CLASS (empathy_contact_parent_class)->finalize (object);
 }
@@ -484,8 +510,8 @@ empathy_contact_set_presence_message (EmpathyContact *contact,
 
   if (priv->persona != NULL)
     {
-      folks_presence_set_presence_message (FOLKS_PRESENCE (priv->persona),
-          message);
+      folks_presence_owner_set_presence_message (
+          FOLKS_PRESENCE_OWNER (priv->persona), message);
     }
 }
 
@@ -703,12 +729,12 @@ empathy_contact_set_alias (EmpathyContact *contact,
 
   /* Set the alias on the persona if possible */
   persona = empathy_contact_get_persona (contact);
-  if (persona != NULL && FOLKS_IS_ALIAS (persona))
+  if (persona != NULL && FOLKS_IS_ALIASABLE (persona))
     {
       DEBUG ("Setting alias for contact %s to %s",
           empathy_contact_get_id (contact), alias);
 
-      folks_alias_set_alias (FOLKS_ALIAS (persona), alias);
+      folks_aliasable_set_alias (FOLKS_ALIASABLE (persona), alias);
     }
 
   if (tp_strdiff (alias, priv->alias))
@@ -726,10 +752,10 @@ groups_change_group_cb (GObject *source,
     GAsyncResult *result,
     gpointer user_data)
 {
-  FolksGroups *groups = FOLKS_GROUPS (source);
+  FolksGroupable *groupable = FOLKS_GROUPABLE (source);
   GError *error = NULL;
 
-  folks_groups_change_group_finish (groups, result, &error);
+  folks_groupable_change_group_finish (groupable, result, &error);
   if (error != NULL)
     {
       g_warning ("failed to change group: %s", error->message);
@@ -753,8 +779,8 @@ empathy_contact_change_group (EmpathyContact *contact, const gchar *group,
   persona = empathy_contact_get_persona (contact);
   if (persona != NULL)
     {
-      if (FOLKS_IS_GROUPS (persona))
-        folks_groups_change_group (FOLKS_GROUPS (persona), group, is_member,
+      if (FOLKS_IS_GROUPABLE (persona))
+        folks_groupable_change_group (FOLKS_GROUPABLE (persona), group, is_member,
           groups_change_group_cb, contact);
       return;
     }
@@ -918,7 +944,7 @@ empathy_contact_set_persona (EmpathyContact *contact,
   /* Set the persona's groups */
   if (priv->groups != NULL)
     {
-      folks_groups_set_groups (FOLKS_GROUPS (persona), priv->groups);
+      folks_groupable_set_groups (FOLKS_GROUPABLE (persona), priv->groups);
       g_hash_table_destroy (priv->groups);
       priv->groups = NULL;
     }
@@ -965,7 +991,10 @@ empathy_contact_get_presence_message (EmpathyContact *contact)
   priv = GET_PRIV (contact);
 
   if (priv->persona != NULL)
-    return folks_presence_get_presence_message (FOLKS_PRESENCE (priv->persona));
+    {
+      return folks_presence_owner_get_presence_message (
+          FOLKS_PRESENCE_OWNER (priv->persona));
+    }
 
   return NULL;
 }
@@ -1395,6 +1424,31 @@ empathy_contact_set_location (EmpathyContact *contact,
   g_object_notify (G_OBJECT (contact), "location");
 }
 
+const gchar * const *
+empathy_contact_get_client_types (EmpathyContact *contact)
+{
+  EmpathyContactPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
+
+  priv = GET_PRIV (contact);
+
+  return (const gchar * const *) priv->client_types;
+}
+
+static void
+contact_set_client_types (EmpathyContact *contact,
+    const gchar * const *client_types)
+{
+  EmpathyContactPriv *priv = GET_PRIV (contact);
+
+  if (priv->client_types != NULL)
+    g_strfreev (priv->client_types);
+
+  priv->client_types = g_strdupv ((gchar **) client_types);
+  g_object_notify (G_OBJECT (contact), "client-types");
+}
+
 /**
  * empathy_contact_equal:
  * @contact1: an #EmpathyContact
@@ -1737,42 +1791,123 @@ empathy_contact_dup_from_tp_contact (TpContact *tp_contact)
 }
 
 static int
-presence_sort_func (EmpathyContact *a, EmpathyContact *b)
+presence_cmp_func (EmpathyContact *a,
+    EmpathyContact *b)
 {
-  FolksPresence *presence_a, *presence_b;
+  FolksPresenceOwner *presence_a, *presence_b;
 
-  presence_a = FOLKS_PRESENCE (empathy_contact_get_persona (a));
-  presence_b = FOLKS_PRESENCE (empathy_contact_get_persona (b));
+  presence_a = FOLKS_PRESENCE_OWNER (empathy_contact_get_persona (a));
+  presence_b = FOLKS_PRESENCE_OWNER (empathy_contact_get_persona (b));
 
   /* We negate the result because we're sorting in reverse order (i.e. such that
    * the Personas with the highest presence are at the beginning of the list. */
-  return -folks_presence_typecmp (folks_presence_get_presence_type (presence_a),
-      folks_presence_get_presence_type (presence_b));
+  return -folks_presence_owner_typecmp (
+      folks_presence_owner_get_presence_type (presence_a),
+      folks_presence_owner_get_presence_type (presence_b));
 }
 
-/* Sort by presence as with presence_sort_func(), but if the two contacts have
+static gint
+voip_cmp_func (EmpathyContact *a,
+    EmpathyContact *b)
+{
+  gboolean has_audio_a, has_audio_b;
+  gboolean has_video_a, has_video_b;
+
+  has_audio_a = empathy_contact_can_voip_audio (a);
+  has_audio_b = empathy_contact_can_voip_audio (b);
+  has_video_a = empathy_contact_can_voip_video (a);
+  has_video_b = empathy_contact_can_voip_video (b);
+
+  /* First check video */
+  if (has_video_a == has_video_b)
+    {
+      /* Use audio to to break tie */
+      if (has_audio_a == has_audio_b)
+        return 0;
+      else if (has_audio_a)
+        return -1;
+      else
+        return 1;
+    }
+  else if (has_video_a)
+    return -1;
+  else
+    return 1;
+}
+
+static gint
+ft_cmp_func (EmpathyContact *a,
+    EmpathyContact *b)
+{
+  gboolean can_send_files_a, can_send_files_b;
+
+  can_send_files_a = empathy_contact_can_send_files (a);
+  can_send_files_b = empathy_contact_can_send_files (b);
+
+  if (can_send_files_a == can_send_files_b)
+    return 0;
+  else if (can_send_files_a)
+    return -1;
+  else
+    return 1;
+}
+
+static gint
+rfb_stream_tube_cmp_func (EmpathyContact *a,
+    EmpathyContact *b)
+{
+  gboolean rfb_a, rfb_b;
+
+  rfb_a = empathy_contact_can_use_rfb_stream_tube (a);
+  rfb_b = empathy_contact_can_use_rfb_stream_tube (b);
+
+  if (rfb_a == rfb_b)
+    return 0;
+  else if (rfb_a)
+    return -1;
+  else
+    return 1;
+}
+
+/* Sort by presence as with presence_cmp_func(), but if the two contacts have
  * the same presence, prefer the one which can do both audio *and* video calls,
  * over the one which can only do one of the two. */
 static int
 voip_sort_func (EmpathyContact *a, EmpathyContact *b)
 {
-  gboolean has_audio_video_a, has_audio_video_b;
-  gint presence_sort = presence_sort_func (a, b);
+  gint presence_sort = presence_cmp_func (a, b);
 
   if (presence_sort != 0)
     return presence_sort;
 
-  has_audio_video_a = empathy_contact_can_voip_audio (a) &&
-      empathy_contact_can_voip_video (a);
-  has_audio_video_b = empathy_contact_can_voip_audio (b) &&
-      empathy_contact_can_voip_video (b);
+  return voip_cmp_func (a, b);
+}
 
-  if (has_audio_video_a && !has_audio_video_b)
-    return -1;
-  else if (!has_audio_video_a && has_audio_video_b)
-    return 1;
-  else
-    return 0;
+/* Sort by presence as with presence_cmp_func() and then break ties using the
+ * most "capable" individual. So users will be able to pick more actions on
+ * the contact in the "Contact" menu of the chat window. */
+static gint
+chat_sort_func (EmpathyContact *a,
+    EmpathyContact *b)
+{
+  gint result;
+
+  result = presence_cmp_func (a, b);
+  if (result != 0)
+    return result;
+
+  /* Prefer individual supporting file transfer */
+  result = ft_cmp_func (a, b);
+  if (result != 0)
+    return result;
+
+  /* Check audio/video capabilities */
+  result = voip_cmp_func (a, b);
+  if (result != 0)
+    return result;
+
+  /* Check 'Share my destop' feature */
+  return rfb_stream_tube_cmp_func (a, b);
 }
 
 static GCompareFunc
@@ -1784,11 +1919,12 @@ get_sort_func_for_action (EmpathyActionType action_type)
       case EMPATHY_ACTION_VIDEO_CALL:
         return (GCompareFunc) voip_sort_func;
       case EMPATHY_ACTION_CHAT:
+        return (GCompareFunc) chat_sort_func;
       case EMPATHY_ACTION_VIEW_LOGS:
       case EMPATHY_ACTION_SEND_FILE:
       case EMPATHY_ACTION_SHARE_MY_DESKTOP:
       default:
-        return (GCompareFunc) presence_sort_func;
+        return (GCompareFunc) presence_cmp_func;
     }
 }
 
