@@ -30,7 +30,6 @@
 #include "empathy-tp-chat.h"
 #include "empathy-tp-contact-factory.h"
 #include "empathy-contact-list.h"
-#include "empathy-dispatcher.h"
 #include "empathy-marshal.h"
 #include "empathy-time.h"
 #include "empathy-utils.h"
@@ -338,8 +337,7 @@ tp_chat_received_cb (TpChannel   *channel,
 		return;
 	}
 
-	DEBUG ("Message received from channel %s: %s",
-		tp_proxy_get_object_path (channel), message_body);
+ 	DEBUG ("Message received: %s", message_body);
 
 	if (message_flags & TP_CHANNEL_TEXT_MESSAGE_FLAG_NON_TEXT_CONTENT &&
 	    !tp_strdiff (message_body, "")) {
@@ -1090,7 +1088,7 @@ tp_chat_got_renamed_contacts_cb (TpConnection            *connection,
 
 	if (priv->user == old) {
 		/* We change our nick */
-		tp_clear_object (&priv->user);
+		g_object_unref (priv->user);
 		priv->user = g_object_ref (new);
 	}
 
@@ -1273,6 +1271,9 @@ tp_chat_constructor (GType                  type,
 			  G_CALLBACK (tp_chat_invalidated_cb),
 			  chat, 0);
 
+	g_assert (tp_proxy_is_prepared (priv->connection,
+		TP_CONNECTION_FEATURE_CAPABILITIES));
+
 	if (tp_proxy_has_interface_by_id (priv->channel,
 					  TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP)) {
 		const TpIntSet *members;
@@ -1296,8 +1297,9 @@ tp_chat_constructor (GType                  type,
 		tp_g_signal_connect_object (priv->channel, "group-members-changed",
 			G_CALLBACK (tp_chat_group_members_changed_cb), chat, 0);
 	} else {
-		EmpathyDispatcher *dispatcher = empathy_dispatcher_dup_singleton ();
-		GList *list, *ptr;
+		TpCapabilities *caps;
+		GPtrArray *classes;
+		guint i;
 
 		/* Get the self contact from the connection's self handle */
 		handle = tp_connection_get_self_handle (priv->connection);
@@ -1311,13 +1313,13 @@ tp_chat_constructor (GType                  type,
 			handle, tp_chat_got_remote_contact_cb,
 			NULL, NULL, chat);
 
-		list = empathy_dispatcher_find_requestable_channel_classes (
-			dispatcher, priv->connection,
-			tp_channel_get_channel_type (priv->channel),
-			TP_UNKNOWN_HANDLE_TYPE, NULL);
+		caps = tp_connection_get_capabilities (priv->connection);
+		g_assert (caps != NULL);
 
-		for (ptr = list; ptr; ptr = ptr->next) {
-			GValueArray *array = ptr->data;
+		classes = tp_capabilities_get_channel_classes (caps);
+
+		for (i = 0; i < classes->len; i++) {
+			GValueArray *array = g_ptr_array_index (classes, i);
 			const char **oprops = g_value_get_boxed (
 				g_value_array_get_nth (array, 1));
 
@@ -1326,9 +1328,6 @@ tp_chat_constructor (GType                  type,
 				break;
 			}
 		}
-
-		g_list_free (list);
-		g_object_unref (dispatcher);
 	}
 
 	if (tp_proxy_has_interface_by_id (priv->channel,
@@ -1960,3 +1959,12 @@ empathy_tp_chat_is_invited (EmpathyTpChat *self,
 	return tp_channel_group_get_local_pending_info (priv->channel, self_handle,
 		inviter, NULL, NULL);
 }
+
+TpChannelChatState
+empathy_tp_chat_get_chat_state (EmpathyTpChat *chat,
+			    EmpathyContact *contact)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (chat);
+	return tp_channel_get_chat_state (priv->channel,
+		empathy_contact_get_handle (contact));
+}
