@@ -666,7 +666,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	else if (empathy_chat_get_nb_unread_messages (chat) > 0) {
 		icon_name = EMPATHY_IMAGE_MESSAGE;
 	}
-	else if (empathy_chat_is_composing (chat)) {
+	else if (remote_contact && empathy_chat_is_composing (chat)) {
 		icon_name = EMPATHY_IMAGE_TYPING;
 	}
 	else if (remote_contact) {
@@ -711,7 +711,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 				      _("Topic:"), subject);
 	}
 
-	if (empathy_chat_is_composing (chat)) {
+	if (remote_contact && empathy_chat_is_composing (chat)) {
 		append_markup_printf (tooltip, "\n%s", _("Typing a message."));
 	}
 
@@ -1454,13 +1454,14 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 			chat_window_set_highlight_room_labels (chat);
 		}
 
-		empathy_sound_manager_play (priv->sound_mgr, GTK_WIDGET (priv->dialog),
-		    EMPATHY_SOUND_MESSAGE_INCOMING);
+		/* Pending messages have already been displayed and notified in the
+		* approver, so we don't display a notification and play a sound for those */
+		if (!pending) {
+			empathy_sound_manager_play (priv->sound_mgr, GTK_WIDGET (priv->dialog),
+				    EMPATHY_SOUND_MESSAGE_INCOMING);
 
-		/* Pending messages have already been displayed in the approver, so we don't
-		* display a notification for those. */
-		if (!pending)
 			chat_window_show_or_update_notification (window, message, chat);
+		}
 	}
 
 	/* update the number of unread messages and the window icon */
