@@ -29,24 +29,21 @@
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/simple-approver.h>
 
-#include <telepathy-yell/telepathy-yell.h>
-
-#include <libempathy/empathy-idle.h>
-
-#include <libempathy/empathy-channel-factory.h>
+#include <libempathy/empathy-presence-manager.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-tp-chat.h>
-#include <libempathy/empathy-tp-streamed-media.h>
+#include <libempathy/empathy-tp-call.h>
 #include <libempathy/empathy-tp-file.h>
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-gsettings.h>
 
 #include <extensions/extensions.h>
 
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-contact-dialogs.h>
-#include <libempathy-gtk/empathy-sound.h>
+#include <libempathy-gtk/empathy-sound-manager.h>
 
 #include "empathy-event-manager.h"
 #include "empathy-main-window.h"
@@ -58,8 +55,6 @@
 
 #define NOTIFICATION_TIMEOUT 2 /* seconds */
 
-#define ACCEPT_WITHOUT_VIDEO 1
-
 /* The time interval in milliseconds between 2 incoming rings */
 #define MS_BETWEEN_RING 500
 
@@ -88,6 +83,11 @@ typedef struct {
   GSList *approvals;
 
   gint ringing;
+
+  GSettings *gsettings_notif;
+  GSettings *gsettings_ui;
+
+  EmpathySoundManager *sound_mgr;
 } EmpathyEventManagerPriv;
 
 typedef struct _EventPriv EventPriv;
@@ -156,8 +156,6 @@ event_manager_approval_free (EventManagerApproval *approval)
   g_slice_free (EventManagerApproval, approval);
 }
 
-static void event_remove (EventPriv *event);
-
 static void
 event_free (EventPriv *event)
 {
@@ -186,6 +184,14 @@ event_remove (EventPriv *event)
   event_free (event);
 }
 
+void
+empathy_event_remove (EmpathyEvent *event_public)
+{
+  EventPriv *event = (EventPriv *) event_public;
+
+  event_remove (event);
+}
+
 static gboolean
 autoremove_event_timeout_cb (EventPriv *event)
 {
@@ -195,19 +201,12 @@ autoremove_event_timeout_cb (EventPriv *event)
 }
 
 static gboolean
-display_notify_area (void)
-
+display_notify_area (EmpathyEventManager *self)
 {
-  GSettings *gsettings;
-  gboolean result;
+  EmpathyEventManagerPriv *priv = GET_PRIV (self);
 
-  gsettings = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
-
-  result = g_settings_get_boolean (gsettings,
+  return g_settings_get_boolean (priv->gsettings_ui,
       EMPATHY_PREFS_UI_EVENTS_NOTIFY_AREA);
-  g_object_unref (gsettings);
-
-  return result;
 }
 
 static void
@@ -233,7 +232,6 @@ event_manager_add (EmpathyEventManager *manager,
   event->public.header = g_strdup (header);
   event->public.message = g_strdup (message);
   event->public.must_ack = (func != NULL);
-  event->public.handler_instance = approval->handler_instance;
   event->inhibit = FALSE;
   event->func = func;
   event->user_data = user_data;
@@ -243,7 +241,7 @@ event_manager_add (EmpathyEventManager *manager,
   DEBUG ("Adding event %p", event);
   priv->events = g_slist_prepend (priv->events, event);
 
-  if (!display_notify_area ())
+  if (!display_notify_area (manager))
     {
       /* Don't fire the 'event-added' signal as we activate the event now */
       if (approval != NULL)
@@ -406,16 +404,9 @@ reject_channel_claim_cb (GObject *source,
       goto out;
     }
 
-  if (EMPATHY_IS_TP_STREAMED_MEDIA (user_data))
+  if (EMPATHY_IS_TP_CALL (user_data))
     {
-      empathy_tp_streamed_media_close (user_data);
-    }
-  else if (TPY_IS_CALL_CHANNEL (user_data))
-    {
-      tpy_call_channel_hangup_async (user_data,
-          TPY_CALL_STATE_CHANGE_REASON_USER_REQUESTED,
-          "", "", NULL, NULL);
-      tp_channel_close_async (user_data, NULL, NULL);
+      empathy_tp_call_close (user_data);
     }
   else if (EMPATHY_IS_TP_CHAT (user_data))
     {
@@ -482,19 +473,13 @@ event_manager_call_window_confirmation_dialog_response_cb (GtkDialog *dialog,
   gtk_widget_destroy (approval->dialog);
   approval->dialog = NULL;
 
-  if (response == GTK_RESPONSE_ACCEPT)
+  if (response != GTK_RESPONSE_ACCEPT)
     {
-      event_manager_approval_approve (approval);
-    }
-  else if (response == ACCEPT_WITHOUT_VIDEO)
-    {
-      tpy_call_channel_send_video (TPY_CALL_CHANNEL (approval->main_channel),
-        FALSE);
-      event_manager_approval_approve (approval);
+      reject_approval (approval);
     }
   else
     {
-      reject_approval (approval);
+      event_manager_approval_approve (approval);
     }
 }
 
@@ -504,9 +489,9 @@ event_channel_process_voip_func (EventPriv *event)
   GtkWidget *dialog;
   GtkWidget *button;
   GtkWidget *image;
+  EmpathyTpCall *call;
   gboolean video;
   gchar *title;
-  EmpathyEventType etype = event->public.type;
 
   if (event->approval->dialog != NULL)
     {
@@ -514,23 +499,9 @@ event_channel_process_voip_func (EventPriv *event)
       return;
     }
 
-  if (etype == EMPATHY_EVENT_TYPE_VOIP)
-    {
-      EmpathyTpStreamedMedia *call;
-      call = EMPATHY_TP_STREAMED_MEDIA (event->approval->handler_instance);
-      video = empathy_tp_streamed_media_has_initial_video (call);
-    }
-  else if (etype == EMPATHY_EVENT_TYPE_CALL)
-    {
-      TpyCallChannel *call;
-      call = TPY_CALL_CHANNEL (event->approval->handler_instance);
-      g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
-    }
-  else
-    {
-      g_warning ("Unknown event type: %d", event->public.type);
-      return;
-    }
+  call = EMPATHY_TP_CALL (event->approval->handler_instance);
+
+  video = empathy_tp_call_has_initial_video (call);
 
   dialog = gtk_message_dialog_new (NULL, 0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
@@ -572,26 +543,15 @@ event_channel_process_voip_func (EventPriv *event)
     GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
-  if (video && etype == EMPATHY_EVENT_TYPE_CALL)
-    {
-      button = gtk_dialog_add_button (GTK_DIALOG (dialog),
-        _("_Answer"), ACCEPT_WITHOUT_VIDEO);
-
-      image = gtk_image_new_from_icon_name ("call-start",
-        GTK_ICON_SIZE_BUTTON);
-      gtk_button_set_image (GTK_BUTTON (button), image);
-    }
-
   button = gtk_dialog_add_button (GTK_DIALOG (dialog),
-    video ? _("_Answer with video") : _("_Answer"), GTK_RESPONSE_ACCEPT);
+      _("_Answer"), GTK_RESPONSE_ACCEPT);
 
-  image = gtk_image_new_from_icon_name ("call-start",
-        GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_icon_name ("call-start", GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
   g_signal_connect (dialog, "response",
-    G_CALLBACK (event_manager_call_window_confirmation_dialog_response_cb),
-    event->approval);
+      G_CALLBACK (event_manager_call_window_confirmation_dialog_response_cb),
+      event->approval);
 
   gtk_widget_show (dialog);
 
@@ -609,6 +569,7 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
   const gchar     *msg;
   TpChannel       *channel;
   EventPriv       *event;
+  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
 
   /* try to update the event if it's referring to a chat which is already in the
    * queue. */
@@ -628,7 +589,8 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
         EMPATHY_EVENT_TYPE_CHAT, EMPATHY_IMAGE_NEW_MESSAGE, header, msg,
         approval, event_text_channel_process_func, NULL);
 
-  empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
+  empathy_sound_manager_play (priv->sound_mgr, window,
+      EMPATHY_SOUND_CONVERSATION_NEW);
 
   g_object_unref (window);
 }
@@ -645,12 +607,12 @@ event_manager_approval_done (EventManagerApproval *approval)
 
       channel_type = tp_channel_get_channel_type_id (approval->main_channel);
 
-      if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA ||
-          channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL)
+      if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
         {
           priv->ringing--;
           if (priv->ringing == 0)
-            empathy_sound_stop (EMPATHY_SOUND_PHONE_INCOMING);
+            empathy_sound_manager_stop (priv->sound_mgr,
+                EMPATHY_SOUND_PHONE_INCOMING);
         }
     }
 
@@ -682,91 +644,18 @@ cdo_invalidated_cb (TpProxy *cdo,
   event_manager_approval_done (approval);
 }
 
-static void
-event_manager_call_state_changed_cb (TpyCallChannel *call,
-  TpyCallState state,
-  TpyCallFlags flags,
-   const GValueArray *call_state_reason,
-  GHashTable *call_state_details,
-  EventManagerApproval *approval)
-{
-  if (state == TPY_CALL_STATE_ENDED)
-    {
-      DEBUG ("Call ended, seems we missed it :/");
-      reject_approval (approval);
-    }
-}
-
-static void
-event_manager_call_channel_got_contact_cb (TpConnection *connection,
-                                 EmpathyContact *contact,
-                                 const GError *error,
-                                 gpointer user_data,
-                                 GObject *object)
-{
-  EventManagerApproval *approval = (EventManagerApproval *) user_data;
-  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
-  GtkWidget *window;
-  TpyCallChannel *call;
-  gchar *header;
-  gboolean video;
-
-  call = TPY_CALL_CHANNEL (approval->handler_instance);
-
-  if (error != NULL)
-    {
-      DEBUG ("Can't get the contact for the call.. Rejecting?");
-      reject_approval (approval);
-      return;
-    }
-
-  if (tpy_call_channel_get_state (call, NULL, NULL) == TPY_CALL_STATE_ENDED)
-    {
-      DEBUG ("Call already ended, seems we missed it :/");
-      reject_approval (approval);
-      return;
-    }
-
-  approval->handler = g_signal_connect (call, "state-changed",
-    G_CALLBACK (event_manager_call_state_changed_cb), approval);
-
-  window = empathy_main_window_dup ();
-  approval->contact = g_object_ref (contact);
-
-  g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
-
-  header = g_strdup_printf (
-    video ? _("Incoming video call from %s") :_("Incoming call from %s"),
-    empathy_contact_get_alias (approval->contact));
-
-  event_manager_add (approval->manager, NULL,
-      approval->contact, EMPATHY_EVENT_TYPE_CALL,
-      video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
-      header, NULL, approval,
-      event_channel_process_voip_func, NULL);
-
-  g_free (header);
-
-  priv->ringing++;
-  if (priv->ringing == 1)
-    empathy_sound_start_playing (window,
-        EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
-
-  g_object_unref (window);
-}
-
 static void
 event_manager_media_channel_got_contact (EventManagerApproval *approval)
 {
   EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
   GtkWidget *window = empathy_main_window_dup ();
   gchar *header;
-  EmpathyTpStreamedMedia *call;
+  EmpathyTpCall *call;
   gboolean video;
 
-  call = EMPATHY_TP_STREAMED_MEDIA (approval->handler_instance);
+  call = EMPATHY_TP_CALL (approval->handler_instance);
 
-  video = empathy_tp_streamed_media_has_initial_video (call);
+  video = empathy_tp_call_has_initial_video (call);
 
   header = g_strdup_printf (
     video ? _("Incoming video call from %s") :_("Incoming call from %s"),
@@ -782,14 +671,14 @@ event_manager_media_channel_got_contact (EventManagerApproval *approval)
 
   priv->ringing++;
   if (priv->ringing == 1)
-    empathy_sound_start_playing (window,
+    empathy_sound_manager_start_playing (priv->sound_mgr, window,
         EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
 
   g_object_unref (window);
 }
 
 static void
-event_manager_media_channel_contact_changed_cb (EmpathyTpStreamedMedia *call,
+event_manager_media_channel_contact_changed_cb (EmpathyTpCall *call,
   GParamSpec *param, EventManagerApproval *approval)
 {
   EmpathyContact *contact;
@@ -887,6 +776,7 @@ display_invite_room_dialog (EventManagerApproval *approval)
   const gchar *invite_msg;
   gchar *msg;
   TpHandle self_handle;
+  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
 
   self_handle = tp_channel_group_get_self_handle (approval->main_channel);
   tp_channel_group_get_local_pending_info (approval->main_channel, self_handle,
@@ -909,7 +799,8 @@ display_invite_room_dialog (EventManagerApproval *approval)
       EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg, approval,
       event_room_channel_process_func, NULL);
 
-  empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
+  empathy_sound_manager_play (priv->sound_mgr, window,
+      EMPATHY_SOUND_CONVERSATION_NEW);
 
   g_free (msg);
   g_object_unref (window);
@@ -946,6 +837,7 @@ event_manager_ft_got_contact_cb (TpConnection *connection,
   EventManagerApproval *approval = (EventManagerApproval *) user_data;
   GtkWidget *window = empathy_main_window_dup ();
   char *header;
+  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
 
   approval->contact = g_object_ref (contact);
 
@@ -958,7 +850,8 @@ event_manager_ft_got_contact_cb (TpConnection *connection,
       approval, event_channel_process_func, NULL);
 
   /* FIXME better sound for incoming file transfers ?*/
-  empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
+  empathy_sound_manager_play (priv->sound_mgr, window,
+      EMPATHY_SOUND_CONVERSATION_NEW);
 
   g_free (header);
   g_object_unref (window);
@@ -989,7 +882,6 @@ find_main_channel (GList *channels)
       channel_type = tp_channel_get_channel_type_id (channel);
 
       if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA ||
-          channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL ||
           channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER ||
           channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
         return channel;
@@ -1083,8 +975,7 @@ approve_channels (TpSimpleApprover *approver,
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
     {
       EmpathyContact *contact;
-      EmpathyTpStreamedMedia *call = empathy_tp_streamed_media_new (account,
-        channel);
+      EmpathyTpCall *call = empathy_tp_call_new (account, channel);
 
       approval->handler_instance = G_OBJECT (call);
 
@@ -1103,19 +994,6 @@ approve_channels (TpSimpleApprover *approver,
         }
 
     }
-  else if (channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL)
-    {
-      TpyCallChannel *call = TPY_CALL_CHANNEL (channel);
-      const gchar *id;
-
-      approval->handler_instance = g_object_ref (call);
-
-      id = tp_channel_get_identifier (channel);
-
-      empathy_tp_contact_factory_get_from_id (connection, id,
-        event_manager_call_channel_got_contact_cb,
-        approval, NULL, G_OBJECT (self));
-    }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
     {
       TpHandle handle;
@@ -1125,6 +1003,7 @@ approve_channels (TpSimpleApprover *approver,
 
       handle = tp_channel_get_handle (channel, NULL);
 
+      connection = tp_channel_borrow_connection (channel);
       empathy_tp_contact_factory_get_from_handle (connection, handle,
         event_manager_ft_got_contact_cb, approval, NULL, G_OBJECT (self));
     }
@@ -1154,7 +1033,8 @@ out:
 static void
 event_pending_subscribe_func (EventPriv *event)
 {
-  empathy_subscription_dialog_show (event->public.contact, NULL);
+  empathy_subscription_dialog_show (event->public.contact, event->public.header,
+      NULL);
   event_remove (event);
 }
 
@@ -1187,7 +1067,7 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
     }
 
   header = g_strdup_printf (
-      _("%s would like permission to see when you are online"),
+      _("%s would like permission to see when you are available"),
       empathy_contact_get_alias (contact));
 
   if (!EMP_STR_EMPTY (message))
@@ -1209,16 +1089,16 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
     TpConnectionPresenceType previous,
     EmpathyEventManager *manager)
 {
+  EmpathyEventManagerPriv *priv = GET_PRIV (manager);
   TpAccount *account;
   gchar *header = NULL;
-  EmpathyIdle *idle;
-  GSettings *gsettings = g_settings_new (EMPATHY_PREFS_NOTIFICATIONS_SCHEMA);
+  EmpathyPresenceManager *presence_mgr;
   GtkWidget *window = empathy_main_window_dup ();
 
   account = empathy_contact_get_account (contact);
-  idle = empathy_idle_dup_singleton ();
+  presence_mgr = empathy_presence_manager_dup_singleton ();
 
-  if (empathy_idle_account_is_just_connected (idle, account))
+  if (empathy_presence_manager_account_is_just_connected (presence_mgr, account))
     goto out;
 
   if (tp_connection_presence_type_cmp_availability (previous,
@@ -1229,9 +1109,10 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           TP_CONNECTION_PRESENCE_TYPE_OFFLINE) <= 0)
         {
           /* someone is logging off */
-          empathy_sound_play (window, EMPATHY_SOUND_CONTACT_DISCONNECTED);
+          empathy_sound_manager_play (priv->sound_mgr, window,
+              EMPATHY_SOUND_CONTACT_DISCONNECTED);
 
-          if (g_settings_get_boolean (gsettings,
+          if (g_settings_get_boolean (priv->gsettings_notif,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT))
             {
               header = g_strdup_printf ("<b>%s</b>",
@@ -1250,9 +1131,10 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
             TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
         {
           /* someone is logging in */
-          empathy_sound_play (window, EMPATHY_SOUND_CONTACT_CONNECTED);
+          empathy_sound_manager_play (priv->sound_mgr, window,
+              EMPATHY_SOUND_CONTACT_CONNECTED);
 
-          if (g_settings_get_boolean (gsettings,
+          if (g_settings_get_boolean (priv->gsettings_notif,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN))
             {
               header = g_strdup_printf ("<b>%s</b>",
@@ -1267,8 +1149,7 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
   g_free (header);
 
 out:
-  g_object_unref (idle);
-  g_object_unref (gsettings);
+  g_object_unref (presence_mgr);
   g_object_unref (window);
 }
 
@@ -1315,7 +1196,7 @@ event_manager_finalize (GObject *object)
   EmpathyEventManagerPriv *priv = GET_PRIV (object);
 
   if (priv->ringing > 0)
-    empathy_sound_stop (EMPATHY_SOUND_PHONE_INCOMING);
+    empathy_sound_manager_stop (priv->sound_mgr, EMPATHY_SOUND_PHONE_INCOMING);
 
   g_slist_foreach (priv->events, (GFunc) event_free, NULL);
   g_slist_free (priv->events);
@@ -1324,6 +1205,9 @@ event_manager_finalize (GObject *object)
   g_object_unref (priv->contact_manager);
   g_object_unref (priv->approver);
   g_object_unref (priv->auth_approver);
+  g_object_unref (priv->gsettings_notif);
+  g_object_unref (priv->gsettings_ui);
+  g_object_unref (priv->sound_mgr);
 }
 
 static void
@@ -1371,11 +1255,15 @@ empathy_event_manager_init (EmpathyEventManager *manager)
   EmpathyEventManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
     EMPATHY_TYPE_EVENT_MANAGER, EmpathyEventManagerPriv);
   TpDBusDaemon *dbus;
-  EmpathyChannelFactory *factory;
   GError *error = NULL;
 
   manager->priv = priv;
 
+  priv->gsettings_notif = g_settings_new (EMPATHY_PREFS_NOTIFICATIONS_SCHEMA);
+  priv->gsettings_ui = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
+
+  priv->sound_mgr = empathy_sound_manager_dup_singleton ();
+
   priv->contact_manager = empathy_contact_manager_dup_singleton ();
   g_signal_connect (priv->contact_manager, "pendings-changed",
     G_CALLBACK (event_manager_pendings_changed_cb), manager);
@@ -1394,10 +1282,9 @@ empathy_event_manager_init (EmpathyEventManager *manager)
   priv->approver = tp_simple_approver_new (dbus, "Empathy.EventManager", FALSE,
       approve_channels, manager, NULL);
 
-  factory = empathy_channel_factory_new ();
-  tp_base_client_set_channel_factory (priv->approver,
-      TP_CLIENT_CHANNEL_FACTORY (factory));
-  g_object_unref (factory);
+  /* EmpathyTpChat relies on this feature being prepared */
+  tp_base_client_add_connection_features_varargs (priv->approver,
+    TP_CONNECTION_FEATURE_CAPABILITIES, 0);
 
   /* Private text channels */
   tp_base_client_take_approver_filter (priv->approver,
@@ -1428,12 +1315,6 @@ empathy_event_manager_init (EmpathyEventManager *manager)
           TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
         TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
         NULL));
-  tp_base_client_take_approver_filter (priv->approver,
-      tp_asv_new (
-        TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
-          TPY_IFACE_CHANNEL_TYPE_CALL,
-        TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
-        NULL));
 
   /* I don't feel good about doing this, and I'm sorry, but the
    * capabilities connection feature is added earlier because it's
